<?php

namespace Acts\CamdramBundle\Entity;

use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Bridge\Doctrine\Validator\Constraints\UniqueEntity;
use Acts\CamdramBundle\Validator\Constraints\TechieAdvertExpiry;
use Acts\CamdramApiBundle\Configuration\Annotation as Api;
use Gedmo\Mapping\Annotation as Gedmo;
use JMS\Serializer\Annotation as Serializer;

/**
 * TechieAdvert
 *
 * @ORM\Table(name="acts_techies")
 * @ORM\Entity(repositoryClass="Acts\CamdramBundle\Entity\TechieAdvertRepository")
 * @ORM\EntityListeners({"Acts\CamdramBundle\EventListener\TechieAdvertListener" })
 * @UniqueEntity("show")
 * @Api\Feed(name="Camdram.net - Production Team Vacancies", titleField="feed_title",
 *     description="Production Team Vacancies advertised for shows in Cambridge",
 *     template="techie_advert/rss.html.twig")
 * @TechieAdvertExpiry()
 * @Gedmo\Loggable
 * @Serializer\ExclusionPolicy("all")
 * @Api\Link(route="get_techie", params={"identifier": "object.getShow().getSlug()"})
 * @Serializer\XmlRoot("techieadvert")
 */
class TechieAdvert
{
    /**
     * @var int
     *
     * @ORM\Column(name="id", type="integer", nullable=false)
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="IDENTITY")
     * @Serializer\Expose
     * @Serializer\XmlAttribute
     */
    private $id;

    /**
     * @var \Show
     *
     * @ORM\ManyToOne(targetEntity="Show", inversedBy="techie_adverts")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="showid", referencedColumnName="id", onDelete="CASCADE")
     * })
     * @Gedmo\Versioned
     * @Api\Link(embed=true, route="get_show", params={"identifier": "object.getShow().getSlug()"})
     */
    private $show;

    /**
     * @var string
     *
     * @ORM\Column(name="positions", type="text", nullable=false)
     * @Assert\NotBlank()
     * @Gedmo\Versioned
     * @Serializer\Expose
     */
    private $positions;

    /**
     * @var string
     *
     * @ORM\Column(name="contact", type="text", nullable=false)
     * @Assert\NotBlank()
     * @Gedmo\Versioned
     * @Serializer\Expose
     */
    private $contact;

    /**
     * @var bool
     *
     * @ORM\Column(name="deadline", type="boolean", nullable=false)
     * @Gedmo\Versioned
     * @Serializer\Expose
     */
    private $deadline;

    /**
     * @var \DateTime
     *
     * @ORM\Column(name="expiry", type="datetime", nullable=false)
     * @Assert\Type("\DateTimeInterface")
     * @Gedmo\Versioned
     * @Serializer\Expose
     */
    private $expiry;

    /**
     * @var bool
     *
     * @ORM\Column(name="display", type="boolean", nullable=false)
     * @Gedmo\Versioned
     */
    private $display = false;

    /**
     * @var bool
     *
     * @ORM\Column(name="remindersent", type="boolean", nullable=false)
     * @Gedmo\Versioned
     */
    private $reminder_sent = false;

    /**
     * @var string
     *
     * @ORM\Column(name="techextra", type="text", nullable=false)
     * @Assert\Length(max=8000)
     * @Gedmo\Versioned
     * @Serializer\Expose
     */
    private $tech_extra = '';

    /**
     * @var \DateTime
     *
     * @ORM\Column(name="lastupdated", type="datetime", nullable=false)
     * @Gedmo\Versioned
     * @Serializer\Expose
     * @Serializer\XmlElement(cdata=false)
     */
    private $last_updated;

    public function __construct()
    {
        $this->setUpdatedAt(new \DateTime());
        $this->setExpiry(new \DateTime('+10 days'));
    }

    /**
     * Get id
     *
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set positions
     *
     * @param string $positions
     *
     * @return TechieAdvert
     */
    public function setPositions($positions)
    {
        $this->positions = $positions;

        return $this;
    }

    /**
     * Get positions
     *
     * @return string
     */
    public function getPositions()
    {
        return $this->positions;
    }

    public function getPositionsOneLine()
    {
        return preg_replace('/\r?\n/', ', ', $this->positions);
    }

    /**
     * Set contact
     *
     * @param string $contact
     *
     * @return TechieAdvert
     */
    public function setContact($contact)
    {
        $this->contact = $contact;

        return $this;
    }

    /**
     * Get contact
     *
     * @return string
     */
    public function getContact()
    {
        return $this->contact;
    }

    /**
     * Set deadline
     *
     * @param bool $deadline
     *
     * @return TechieAdvert
     */
    public function setDeadline($deadline)
    {
        $this->deadline = $deadline;

        return $this;
    }

    /**
     * Get deadline
     *
     * @return bool
     */
    public function getDeadline()
    {
        return $this->deadline;
    }

    /**
     * Set expiry
     *
     * @param \DateTime $expiry
     *
     * @return TechieAdvert
     */
    public function setExpiry($expiry)
    {
        $this->expiry = $expiry;

        return $this;
    }

    /**
     * Get expiry
     *
     * @return \DateTime
     */
    public function getExpiry()
    {
        return $this->expiry;
    }

    /**
     * Set display
     *
     * @param bool $display
     *
     * @return TechieAdvert
     */
    public function setDisplay($display)
    {
        $this->display = $display;

        return $this;
    }

    /**
     * Get display
     *
     * @return bool
     */
    public function getDisplay()
    {
        return $this->display;
    }

    /**
     * Set reminder_sent
     *
     * @param bool $reminderSent
     *
     * @return TechieAdvert
     */
    public function setReminderSent($reminderSent)
    {
        $this->reminder_sent = $reminderSent;

        return $this;
    }

    /**
     * Get reminder_sent
     *
     * @return bool
     */
    public function getReminderSent()
    {
        return $this->reminder_sent;
    }

    /**
     * Set tech_extra
     *
     * @param string $techExtra
     *
     * @return TechieAdvert
     */
    public function setTechExtra($techExtra)
    {
        if (!($techExtra)) {
            $this->tech_extra = '';
        } else {
            $this->tech_extra = $techExtra;
        }

        return $this;
    }

    /**
     * Get tech_extra
     *
     * @return string
     */
    public function getTechExtra()
    {
        return $this->tech_extra;
    }

    /**
     * Set last_updated
     *
     * @param \DateTime $lastUpdated
     *
     * @return TechieAdvert
     */
    public function setUpdatedAt($lastUpdated)
    {
        $this->last_updated = $lastUpdated;

        return $this;
    }

    /**
     * Get last_updated
     *
     * @return \DateTime
     */
    public function getUpdatedAt()
    {
        return $this->last_updated;
    }

    public function getCreatedAt()
    {
        return $this->last_updated;
    }

    /**
     * Set show
     *
     * @param \Acts\CamdramBundle\Entity\Show $showId
     *
     * @return TechieAdvert
     */
    public function setShow(\Acts\CamdramBundle\Entity\Show $show = null)
    {
        $this->show = $show;

        return $this;
    }

    /**
     * Get show
     *
     * @return \Acts\CamdramBundle\Entity\Show
     */
    public function getShow()
    {
        return $this->show;
    }

    public function getFeedTitle()
    {
        return $this->getShow()->getName(). ' - last updated '.$this->getUpdatedAt()->format('D, j M Y H:i T');
    }

    public function getSlug()
    {
        return $this->getShow()->getSlug();
    }

    /**
     * Set last_updated
     *
     * @param \DateTime $lastUpdated
     *
     * @return TechieAdvert
     */
    public function setLastUpdated($lastUpdated)
    {
        $this->last_updated = $lastUpdated;

        return $this;
    }

    /**
     * Get last_updated
     *
     * @return \DateTime
     */
    public function getLastUpdated()
    {
        return $this->last_updated;
    }
}
